@@ -1,24 +1,24 @@
 /* -*- mode: c++; indent-tabs-mode: nil -*- */
 /*
-  QoreOracleConnection.cpp
+    QoreOracleConnection.cpp
 
-  Qore Programming Language
+    Qore Programming Language
 
-  Copyright (C) 2003 - 2017 Qore Technologies, s.r.o.
+    Copyright (C) 2003 - 2022 Qore Technologies, s.r.o.
 
-  This library is free software; you can redistribute it and/or
-  modify it under the terms of the GNU Lesser General Public
-  License as published by the Free Software Foundation; either
-  version 2.1 of the License, or (at your option) any later version.
+    This library is free software; you can redistribute it and/or
+    modify it under the terms of the GNU Lesser General Public
+    License as published by the Free Software Foundation; either
+    version 2.1 of the License, or (at your option) any later version.
 
-  This library is distributed in the hope that it will be useful,
-  but WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  Lesser General Public License for more details.
+    This library is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+    Lesser General Public License for more details.
 
-  You should have received a copy of the GNU Lesser General Public
-  License along with this library; if not, write to the Free Software
-  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+    You should have received a copy of the GNU Lesser General Public
+    License along with this library; if not, write to the Free Software
+    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
 
 #include "oracle.h"
@@ -31,17 +31,11 @@ static char session_sql[] = "alter session set nls_numeric_characters = \". \"";
 
 QoreOracleConnection::QoreOracleConnection(Datasource &n_ds, ExceptionSink *xsink)
   : errhp(0), svchp(0), srvhp(0), usrhp(0), ocilib_cn(0), ds(n_ds), ocilib_init(false),
-#ifdef _QORE_HAS_FIND_CREATE_TIMEZONE
     server_tz(currentTZ()),
-#endif
     number_support(OPT_NUM_DEFAULT) {
    // locking is done on the level above with the Datasource class
 
-#ifdef QORE_HAS_DATASOURCE_PORT
    int port = ds.getPort();
-#else
-   int port = 0;
-#endif
 
    if (port)
       cstr.sprintf("//%s:%d/%s", ds.getHostName(), port, ds.getDBName());
@@ -64,8 +58,7 @@ QoreOracleConnection::QoreOracleConnection(Datasource &n_ds, ExceptionSink *xsin
 
          // get character set ID
          charsetid = tmpenv.nlsCharSetNameToId(ds.getDBEncoding());
-      }
-      else { // get Oracle character set name from OS character set name
+      } else { // get Oracle character set name from OS character set name
          if (tmpenv.nlsNameMapToOracle(QCS_DEFAULT->getCode(), encoding) != OCI_SUCCESS) {
             xsink->raiseException("DBI:ORACLE:UNKNOWN-CHARACTER-ENCODING", "cannot map default OS encoding '%s' to Oracle character encoding", QCS_DEFAULT->getCode());
             return;
@@ -225,51 +218,146 @@ void QoreOracleConnection::clearCache() {
    OCI_ListClear(&ocilib, ocilib_cn->tinfs);
 }
 
-int QoreOracleConnection::checkerr(sword status, const char *query_name, ExceptionSink *xsink) {
-   sb4 errcode = 0;
+int QoreOracleConnection::checkerr(sword status, const char* query_name, ExceptionSink* xsink, bool* retry) {
+    sb4 errcode = 0;
 
-   //printd(5, "QoreOracleConnection::checkerr(%p, %d, %s, isEvent=%d)\n", errhp, status, query_name ? query_name : "none", *xsink);
-   switch (status) {
-      case OCI_SUCCESS:
-         return 0;
-      case OCI_SUCCESS_WITH_INFO: {
-         text errbuf[512];
+    //printd(5, "QoreOracleConnection::checkerr(%p, %d, %s, isEvent=%d)\n", errhp, status, query_name ? query_name : "none", *xsink);
+    switch (status) {
+        case OCI_SUCCESS:
+            return 0;
 
-         OCIErrorGet((dvoid *)errhp, (ub4) 1, (text *) NULL, &errcode, errbuf, (ub4) sizeof(errbuf), OCI_HTYPE_ERROR);
-         //printd(5, "WARNING: %s returned OCI_SUCCESS_WITH_INFO: %s\n", query_name ? query_name : "<unknown>", remove_trailing_newlines((char *)errbuf));
-         // ignore SUCCESS_WITH_INFO codes
-         return 0;
-      }
+        case OCI_SUCCESS_WITH_INFO: {
+            text errbuf[512];
 
-      case OCI_ERROR: {
-         text errbuf[512];
-         OCIErrorGet((dvoid *)errhp, (ub4) 1, (text *) NULL, &errcode, errbuf, (ub4) sizeof(errbuf), OCI_HTYPE_ERROR);
-         doException(query_name, errbuf, errcode, xsink);
-         break;
-      }
-      case OCI_INVALID_HANDLE:
-         if (query_name)
-            xsink->raiseException("DBI:ORACLE:OCI-INVALID-HANDLE", "%s@%s: %s: an invalid OCI handle was used", ds.getUsername(), ds.getDBName(), query_name);
-         else
-            xsink->raiseException("DBI:ORACLE:OCI-INVALID-HANDLE", "%s@%s: an invalid OCI handle was used", ds.getUsername(), ds.getDBName());
-         break;
-      case OCI_NEED_DATA:
-         xsink->raiseException("DBI:ORACLE:OCI-NEED-DATA", "Oracle OCI error");
-         break;
-      case OCI_NO_DATA:
-         xsink->raiseException("DBI:ORACLE:OCI-NODATA", "Oracle OCI error");
-         break;
-      case OCI_STILL_EXECUTING:
-         xsink->raiseException("DBI:ORACLE:OCI-STILL-EXECUTING", "Oracle OCI error");
-         break;
-      case OCI_CONTINUE:
-         xsink->raiseException("DBI:ORACLE:OCI-CONTINUE", "Oracle OCI error");
-         break;
-      default:
-         xsink->raiseException("DBI:ORACLE:UNKNOWN-ERROR", "unknown OCI error code %d", status);
-         break;
-   }
-   return -1;
+            OCIErrorGet((dvoid *)errhp, (ub4) 1, (text *) NULL, &errcode, errbuf, (ub4) sizeof(errbuf), OCI_HTYPE_ERROR);
+            //printd(5, "WARNING: %s returned OCI_SUCCESS_WITH_INFO: %s\n", query_name ? query_name : "<unknown>", remove_trailing_newlines((char *)errbuf));
+            // ignore SUCCESS_WITH_INFO codes
+            return 0;
+        }
+
+        case OCI_ERROR: {
+            bool retry_flag = handleError(xsink, query_name, (bool)retry);
+            if (retry) {
+                assert(!*retry);
+                *retry = retry_flag;
+            }
+            break;
+        }
+
+        case OCI_INVALID_HANDLE:
+            if (query_name)
+                xsink->raiseException("DBI:ORACLE:OCI-INVALID-HANDLE", "%s@%s: %s: an invalid OCI handle was used", ds.getUsername(), ds.getDBName(), query_name);
+            else
+                xsink->raiseException("DBI:ORACLE:OCI-INVALID-HANDLE", "%s@%s: an invalid OCI handle was used", ds.getUsername(), ds.getDBName());
+            break;
+
+        case OCI_NEED_DATA:
+            xsink->raiseException("DBI:ORACLE:OCI-NEED-DATA", "Oracle OCI error");
+            break;
+
+        case OCI_NO_DATA:
+            xsink->raiseException("DBI:ORACLE:OCI-NODATA", "Oracle OCI error");
+            break;
+
+        case OCI_STILL_EXECUTING:
+            xsink->raiseException("DBI:ORACLE:OCI-STILL-EXECUTING", "Oracle OCI error");
+            break;
+
+        case OCI_CONTINUE:
+            xsink->raiseException("DBI:ORACLE:OCI-CONTINUE", "Oracle OCI error");
+            break;
+
+        default:
+            xsink->raiseException("DBI:ORACLE:UNKNOWN-ERROR", "unknown OCI error code %d", status);
+            break;
+    }
+    return -1;
+}
+
+bool QoreOracleConnection::handleError(ExceptionSink* xsink, const char* who, bool can_retry) {
+    // see if we have a lost connection from the error code
+    int ping = -1;
+
+    // get and save error information
+    sb4 errcode;
+    text errbuf[512];
+    OCIErrorGet((dvoid*)errhp, (ub4)1, (text*)nullptr, &errcode, errbuf, (ub4)sizeof(errbuf), OCI_HTYPE_ERROR);
+    // ORA-03113 and ORA-03114 are lost connections
+    // issue #802: ORA-01041 is only raised due to a bug in Oracle pre 12c
+    if (errcode == 3113 || errcode == 3114 || errcode == 1041)
+        ping = 0;
+
+    //dbg();
+    // see if server is connected
+    if (ping == -1 && ocilib_init) {
+        ExceptionSink xsink2;
+        ping = OCI_Ping(&ocilib, ocilib_cn, &xsink2);
+        // do not allow ping exceptions to be propagated to the caller
+        xsink2.clear();
+    }
+
+    if (!ping) {
+        // if there is at least one SQLStatement active on the connection, there will be a transaction in place
+        // therefore in all cases all statements will be invalidated and closed when we lose a connection
+        // we can only recover a plain "exec" or "select" call
+        if (ds.activeTransaction()) {
+            xsink->raiseException("DBI:ORACLE:TRANSACTION-ERROR", "connection to Oracle database server %s@%s lost " \
+                "while in a transaction; transaction has been lost", ds.getUsername(), ds.getDBName());
+        }
+
+        // reset current statement state while the driver-specific context data is still present
+        for (auto& i : stmt_set) {
+            i->clear(xsink);
+        }
+        // free and reset statement states for all active statements while the driver-specific context data is still present
+        ds.connectionLost(xsink);
+
+        if (can_retry) {
+            // try to reconnect
+            logoff();
+
+            //printd(5, "QoreOracleStatement::execute() about to execute OCILogon() for reconnect (trans: %d)\n", ds->activeTransaction());
+            if (logon(xsink)) {
+                //printd(5, "QoreOracleStatement::execute() conn: %p reconnect failed, marking connection as closed\n", &conn);
+                // free state completely
+                for (auto& i : stmt_set) {
+                    i->reset(xsink);
+                }
+                // close datasource and remove private data
+                ds.connectionAborted(xsink);
+                return false;
+            }
+
+            // clear warnings
+            clearWarnings();
+
+            // don't execute again if any exceptions have occured, including if the connection was aborted while in a transaction
+            if (*xsink) {
+                // close all statements and remove private data but leave datasource open
+                ds.connectionRecovered(xsink);
+                return false;
+            }
+
+            for (auto& i : stmt_set) {
+                // try to recreate the statement context
+                if (i->rebindAbortedConnection(xsink))
+                    return false;
+            }
+
+            return true;
+        }
+
+        for (auto& i : stmt_set) {
+            i->reset(xsink);
+        }
+        // close datasource and remove private data
+        ds.connectionAborted(xsink);
+        return false;
+    }
+
+    //printd(5, "QoreOracleStatement::execute() error, but it's connected; status: %d who: %s\n", status, who);
+    doException(who, errbuf, errcode, xsink);
+    return false;
 }
 
 int QoreOracleConnection::doException(const char *query_name, text errbuf[], sb4 errcode, ExceptionSink *xsink) {
@@ -333,184 +421,176 @@ int QoreOracleConnection::logon(ExceptionSink *xsink) {
    return stmt.exec(session_sql, sizeof(session_sql), xsink);
 }
 
-int QoreOracleConnection::descriptorAlloc(void **descpp, unsigned type, const char *who, ExceptionSink *xsink) {
+int QoreOracleConnection::descriptorAlloc(void** descpp, unsigned type, const char* who, ExceptionSink* xsink) {
    return checkerr(OCIDescriptorAlloc(*env, descpp, type, 0, 0), who, xsink);
 }
 
-int QoreOracleConnection::handleAlloc(void **hndlpp, unsigned type, const char *who, ExceptionSink *xsink) {
+int QoreOracleConnection::handleAlloc(void** hndlpp, unsigned type, const char* who, ExceptionSink* xsink) {
    return checkerr(OCIHandleAlloc(*env, hndlpp, type, 0, 0), who, xsink);
 }
 
-int QoreOracleConnection::commit(ExceptionSink *xsink) {
+int QoreOracleConnection::commit(ExceptionSink* xsink) {
    return checkerr(OCITransCommit(svchp, errhp, (ub4) 0), "QoreOracleConnection:commit()", xsink);
 }
 
-int QoreOracleConnection::rollback(ExceptionSink *xsink) {
+int QoreOracleConnection::rollback(ExceptionSink* xsink) {
    return checkerr(OCITransRollback(svchp, errhp, (ub4) 0), "QoreOracleConnection:rollback()", xsink);
 }
 
 DateTimeNode* QoreOracleConnection::getDate(OCIDate* dt) {
-   sb2 year;
-   ub1 month, day;
-   OCIDateGetDate(dt, &year, &month, &day);
+    sb2 year;
+    ub1 month, day;
+    OCIDateGetDate(dt, &year, &month, &day);
 
-   ub1 hour, minute, second;
-   OCIDateGetTime(dt, &hour, &minute, &second);
+    ub1 hour, minute, second;
+    OCIDateGetTime(dt, &hour, &minute, &second);
 
-#ifdef _QORE_HAS_TIME_ZONES
-   return DateTimeNode::makeAbsolute(getTZ(), year, month, day, hour, minute, second, 0);
-#else
-   return new DateTimeNode(year, month, day, hour, minute, second, 0);
-#endif
+    return DateTimeNode::makeAbsolute(getTZ(), year, month, day, hour, minute, second, 0);
 }
 
 DateTimeNode* QoreOracleConnection::getTimestamp(bool get_tz, OCIDateTime *odt, ExceptionSink *xsink) {
-   //printd(5, "QoreOracleConnection::getTimestamp() using TIMESTAMP handle %p\n", odt);
-   sb2 year;
-   ub1 month, day;
-   if (checkerr(OCIDateTimeGetDate(*env, errhp, odt, &year, &month, &day), "OCIDateTimeGetDate()", xsink))
-      return 0;
+    //printd(5, "QoreOracleConnection::getTimestamp() using TIMESTAMP handle %p\n", odt);
+    sb2 year;
+    ub1 month, day;
+    if (checkerr(OCIDateTimeGetDate(*env, errhp, odt, &year, &month, &day), "OCIDateTimeGetDate()", xsink))
+        return nullptr;
 
-   ub1 hour, minute, second;
-   ub4 ns; // nanoseconds
-   if (checkerr(OCIDateTimeGetTime(*env, errhp, odt, &hour, &minute, &second, &ns), "OCIDateTimeGetTime()", xsink))
-      return 0;
+    ub1 hour, minute, second;
+    ub4 ns; // nanoseconds
+    if (checkerr(OCIDateTimeGetTime(*env, errhp, odt, &hour, &minute, &second, &ns), "OCIDateTimeGetTime()", xsink))
+        return nullptr;
 
-#ifdef _QORE_HAS_TIME_ZONES
-   const AbstractQoreZoneInfo *zone;
-   if (!get_tz)
-      zone = getTZ();
-   else {
-      // try to get time zone from date value
-      // time zone offset, hour and minute
-      sb1 oh = 0, om = 0;
-      sword err = OCIDateTimeGetTimeZoneOffset(*env, errhp, odt, &oh, &om);
-      if (err == OCI_SUCCESS) {
-         //printd(5, "err=%d, oh=%d, om=%d, se=%d\n", err, (int)oh, (int)om, oh * 3600 + om * 60);
-         zone = findCreateOffsetZone(oh * 3600 + om * 60);
-      }
-      else {
-         //printd(5, "QoreOracleConnection::getTimestamp() this=%p time zone retrieval failed (%04d-%02d-%02d %02d:%02d:%02d)\n", this, year, month, day, hour, minute, second);
-         // no time zone info, assume local time
-         zone = getTZ();
-      }
-   }
-   return DateTimeNode::makeAbsolute(zone, year, month, day, hour, minute, second, ns / 1000);
-#else
-   return new DateTimeNode(year, month, day, hour, minute, second, ns / 1000000);
-#endif
+    const AbstractQoreZoneInfo *zone;
+    if (!get_tz)
+        zone = getTZ();
+    else {
+        // try to get time zone from date value
+        // time zone offset, hour and minute
+        sb1 oh = 0, om = 0;
+        sword err = OCIDateTimeGetTimeZoneOffset(*env, errhp, odt, &oh, &om);
+        if (err == OCI_SUCCESS) {
+            //printd(5, "err=%d, oh=%d, om=%d, se=%d\n", err, (int)oh, (int)om, oh * 3600 + om * 60);
+            zone = findCreateOffsetZone(oh * 3600 + om * 60);
+        }
+        else {
+            //printd(5, "QoreOracleConnection::getTimestamp() this=%p time zone retrieval failed (%04d-%02d-%02d %02d:%02d:%02d)\n", this, year, month, day, hour, minute, second);
+            // no time zone info, assume local time
+            zone = getTZ();
+        }
+    }
+    return DateTimeNode::makeAbsolute(zone, year, month, day, hour, minute, second, ns / 1000);
 }
 
 BinaryNode *QoreOracleConnection::readBlob(OCILobLocator *lobp, ExceptionSink *xsink) {
-   // retrieve *LOB data
-   void *dbuf = malloc(LOB_BLOCK_SIZE);
-   ON_BLOCK_EXIT(free, dbuf);
-   ub4 amt = 0;
+    // retrieve *LOB data
+    void *dbuf = malloc(LOB_BLOCK_SIZE);
+    ON_BLOCK_EXIT(free, dbuf);
+    ub4 amt = 0;
 
-   SimpleRefHolder<BinaryNode> b(new BinaryNode);
-   // read LOB data in streaming callback mode
-   if (checkerr(OCILobRead(svchp, errhp, lobp, &amt, 1, dbuf, LOB_BLOCK_SIZE, *b, readBlobCallback, 0, 0), "QoreOracleConnection::readBlob()", xsink))
-      return 0;
-   return b.release();
+    SimpleRefHolder<BinaryNode> b(new BinaryNode);
+    // read LOB data in streaming callback mode
+    if (checkerr(OCILobRead(svchp, errhp, lobp, &amt, 1, dbuf, LOB_BLOCK_SIZE, *b, readBlobCallback, 0, 0), "QoreOracleConnection::readBlob()", xsink))
+        return nullptr;
+    return b.release();
 }
 
 QoreStringNode *QoreOracleConnection::readClob(OCILobLocator *lobp, const QoreEncoding *enc, ExceptionSink *xsink) {
-   void *dbuf = malloc(LOB_BLOCK_SIZE);
-   ON_BLOCK_EXIT(free, dbuf);
-   ub4 amt = 0;
+    void *dbuf = malloc(LOB_BLOCK_SIZE);
+    ON_BLOCK_EXIT(free, dbuf);
+    ub4 amt = 0;
 
-   QoreStringNodeHolder str(new QoreStringNode(enc));
-   // read LOB data in streaming callback mode
-   if (checkerr(OCILobRead(svchp, errhp, lobp, &amt, 1, dbuf, LOB_BLOCK_SIZE, *str, readClobCallback, (ub2)charsetid, 0), "QoreOracleConnection::readClob()", xsink))
-      return 0;
-   return str.release();
+    QoreStringNodeHolder str(new QoreStringNode(enc));
+    // read LOB data in streaming callback mode
+    if (checkerr(OCILobRead(svchp, errhp, lobp, &amt, 1, dbuf, LOB_BLOCK_SIZE, *str, readClobCallback, (ub2)charsetid, 0), "QoreOracleConnection::readClob()", xsink))
+        return nullptr;
+    return str.release();
 }
 
 int QoreOracleConnection::writeLob(OCILobLocator* lobp, void* bufp, oraub8 buflen, bool clob, const char* desc, ExceptionSink* xsink) {
 #ifdef HAVE_OCILOBWRITE2
-   oraub8 amtp = buflen;
-   if (buflen <= LOB_BLOCK_SIZE)
-      return checkerr(OCILobWrite2(svchp, errhp, lobp, &amtp, 0, 1, bufp, buflen, OCI_ONE_PIECE, 0, 0, charsetid, SQLCS_IMPLICIT), desc, xsink);
+    oraub8 amtp = buflen;
+    if (buflen <= LOB_BLOCK_SIZE)
+        return checkerr(OCILobWrite2(svchp, errhp, lobp, &amtp, 0, 1, bufp, buflen, OCI_ONE_PIECE, 0, 0, charsetid, SQLCS_IMPLICIT), desc, xsink);
 
-   // retrieve *LOB data
-   void* dbuf = malloc(LOB_BLOCK_SIZE);
-   ON_BLOCK_EXIT(free, dbuf);
+    // retrieve *LOB data
+    void* dbuf = malloc(LOB_BLOCK_SIZE);
+    ON_BLOCK_EXIT(free, dbuf);
 
-   oraub8 offset = 0;
-   while (true) {
-      ub1 piece;
-      oraub8 len = buflen - offset;
-      if (len > LOB_BLOCK_SIZE) {
-         len = LOB_BLOCK_SIZE;
-         piece = offset ? OCI_NEXT_PIECE : OCI_FIRST_PIECE;
-         //printd(5, "QoreOracleConnection::writeLob() piece = %s\n", offset ? "OCI_NEXT_PIECE" : "OCI_FIRST_PIECE");
-      }
-      else {
-         piece = OCI_LAST_PIECE;
-         //printd(5, "QoreOracleConnection::writeLob() piece = OCI_LAST_PIECE\n");
-      }
+    oraub8 offset = 0;
+    while (true) {
+        ub1 piece;
+        oraub8 len = buflen - offset;
+        if (len > LOB_BLOCK_SIZE) {
+            len = LOB_BLOCK_SIZE;
+            piece = offset ? OCI_NEXT_PIECE : OCI_FIRST_PIECE;
+            //printd(5, "QoreOracleConnection::writeLob() piece = %s\n", offset ? "OCI_NEXT_PIECE" : "OCI_FIRST_PIECE");
+        }
+        else {
+            piece = OCI_LAST_PIECE;
+            //printd(5, "QoreOracleConnection::writeLob() piece = OCI_LAST_PIECE\n");
+        }
 
-      // copy data to buffer
-      memcpy(dbuf, ((char*)bufp) + offset, len);
+        // copy data to buffer
+        memcpy(dbuf, ((char*)bufp) + offset, len);
 
-      sword rc = OCILobWrite2(svchp, errhp, lobp, &amtp, 0, 1, dbuf, len, piece, 0, 0, charsetid, SQLCS_IMPLICIT);
-      //printd(5, "QoreOracleConnection::writeLob() offset: "QLLD" len: "QLLD" amtp: "QLLD" total: "QLLD" rc: %d\n", offset, len, amtp, buflen, (int)rc);
-      if (piece == OCI_LAST_PIECE) {
-         if (rc != OCI_SUCCESS) {
+        sword rc = OCILobWrite2(svchp, errhp, lobp, &amtp, 0, 1, dbuf, len, piece, 0, 0, charsetid, SQLCS_IMPLICIT);
+        //printd(5, "QoreOracleConnection::writeLob() offset: "QLLD" len: "QLLD" amtp: "QLLD" total: "QLLD" rc: %d\n", offset, len, amtp, buflen, (int)rc);
+        if (piece == OCI_LAST_PIECE) {
+            if (rc != OCI_SUCCESS) {
+                checkerr(rc, desc, xsink);
+                return -1;
+            }
+            break;
+        }
+        if (rc != OCI_NEED_DATA) {
             checkerr(rc, desc, xsink);
+            assert(*xsink);
             return -1;
-         }
-         break;
-      }
-      if (rc != OCI_NEED_DATA) {
-         checkerr(rc, desc, xsink);
-         assert(*xsink);
-         return -1;
-      }
-      offset += len;
-   }
+        }
+        offset += len;
+    }
 #else
-   ub4 amtp = buflen;
-   if (buflen <= LOB_BLOCK_SIZE)
-      return checkerr(OCILobWrite(svchp, errhp, lobp, &amtp, 1, bufp, buflen, OCI_ONE_PIECE, 0, 0, charsetid, SQLCS_IMPLICIT), desc, xsink);
+    ub4 amtp = buflen;
+    if (buflen <= LOB_BLOCK_SIZE)
+        return checkerr(OCILobWrite(svchp, errhp, lobp, &amtp, 1, bufp, buflen, OCI_ONE_PIECE, 0, 0, charsetid, SQLCS_IMPLICIT), desc, xsink);
 
-   // retrieve *LOB data
-   void* dbuf = malloc(LOB_BLOCK_SIZE);
-   ON_BLOCK_EXIT(free, dbuf);
+    // retrieve *LOB data
+    void* dbuf = malloc(LOB_BLOCK_SIZE);
+    ON_BLOCK_EXIT(free, dbuf);
 
-   ub4 offset = 0;
-   while (true) {
-      ub1 piece;
-      ub4 len = buflen - offset;
-      if (len > LOB_BLOCK_SIZE) {
-         len = LOB_BLOCK_SIZE;
-         piece = offset ? OCI_NEXT_PIECE : OCI_FIRST_PIECE;
-         //printd(5, "QoreOracleConnection::writeLob() piece = %s\n", offset ? "OCI_NEXT_PIECE" : "OCI_FIRST_PIECE");
-      }
-      else {
-         piece = OCI_LAST_PIECE;
-         //printd(5, "QoreOracleConnection::writeLob() piece = OCI_LAST_PIECE\n");
-      }
+    ub4 offset = 0;
+    while (true) {
+        ub1 piece;
+        ub4 len = buflen - offset;
+        if (len > LOB_BLOCK_SIZE) {
+            len = LOB_BLOCK_SIZE;
+            piece = offset ? OCI_NEXT_PIECE : OCI_FIRST_PIECE;
+            //printd(5, "QoreOracleConnection::writeLob() piece = %s\n", offset ? "OCI_NEXT_PIECE" : "OCI_FIRST_PIECE");
+        }
+        else {
+            piece = OCI_LAST_PIECE;
+            //printd(5, "QoreOracleConnection::writeLob() piece = OCI_LAST_PIECE\n");
+        }
 
-      // copy data to buffer
-      memcpy(dbuf, ((char*)bufp) + offset, len);
+        // copy data to buffer
+        memcpy(dbuf, ((char*)bufp) + offset, len);
 
-      sword rc = OCILobWrite(svchp, errhp, lobp, &amtp, 1, dbuf, len, piece, 0, 0, charsetid, SQLCS_IMPLICIT);
-      //printd(5, "QoreOracleConnection::writeLob() offset: "QLLD" len: "QLLD" amtp: "QLLD" total: "QLLD" rc: %d\n", offset, len, amtp, buflen, (int)rc);
-      if (piece == OCI_LAST_PIECE) {
-         if (rc != OCI_SUCCESS) {
+        sword rc = OCILobWrite(svchp, errhp, lobp, &amtp, 1, dbuf, len, piece, 0, 0, charsetid, SQLCS_IMPLICIT);
+        //printd(5, "QoreOracleConnection::writeLob() offset: "QLLD" len: "QLLD" amtp: "QLLD" total: "QLLD" rc: %d\n", offset, len, amtp, buflen, (int)rc);
+        if (piece == OCI_LAST_PIECE) {
+            if (rc != OCI_SUCCESS) {
+                checkerr(rc, desc, xsink);
+                return -1;
+            }
+            break;
+        }
+        if (rc != OCI_NEED_DATA) {
             checkerr(rc, desc, xsink);
+            assert(*xsink);
             return -1;
-         }
-         break;
-      }
-      if (rc != OCI_NEED_DATA) {
-         checkerr(rc, desc, xsink);
-         assert(*xsink);
-         return -1;
-      }
-      offset += len;
-   }
+        }
+        offset += len;
+    }
 #endif
-   return 0;
+    return 0;
 }
